# encoding: utf-8
# Created by Jeremy Bowman on Thu Feb 20 16:55:15 EST 2014
# Copyright (c) 2014, 2015 Safari Books Online. All rights reserved.
#
# This software may be modified and distributed under the terms
# of the 3-clause BSD license.  See the LICENSE file for details.
"""
Yet Another Django Profiler middleware

A django profiling middeware that tries to consolidate the best features of
the ones that came before it.  Originally described at
http://blog.safariflow.com/2013/11/21/profiling-django-via-middleware/
"""

__version__ = (1, 1, 0)  # Update docs/CHANGELOG.rst if you increment the version
